@@ -41,17 +41,18 @@ from yd_commands.config import (
 from yd_commands.config_keys import *
 from yd_commands.mustache import (
     LAZY_SUBS_WRAPPER,
+    TASK_COUNT_SUB,
+    TASK_GROUP_COUNT_SUB,
     TASK_GROUP_NUMBER_SUB,
     TASK_NUMBER_SUB,
     load_json_file_with_mustache_substitutions,
     load_jsonnet_file_with_mustache_substitutions,
     load_toml_file_with_mustache_substitutions,
 )
-from yd_commands.printing import print_error, print_log
+from yd_commands.printing import print_error, print_log, print_tasks, print_yd_object
 from yd_commands.type_check import (
     check_bool,
     check_dict,
-    check_float,
     check_float_or_int,
     check_int,
     check_list,
@@ -153,21 +154,24 @@ def submit_work_requirement(
     fulfilOnSubmit = check_bool(
         tasks_data.get(FULFIL_ON_SUBMIT, CONFIG_WR.fulfil_on_submit)
     )
-    work_requirement = CLIENT.work_client.add_work_requirement(
-        WorkRequirement(
-            namespace=CONFIG_COMMON.namespace,
-            name=ID,
-            taskGroups=task_groups,
-            tag=CONFIG_COMMON.name_tag,
-            priority=priority,
-            fulfilOnSubmit=fulfilOnSubmit,
+    work_requirement = WorkRequirement(
+        namespace=CONFIG_COMMON.namespace,
+        name=ID,
+        taskGroups=task_groups,
+        tag=CONFIG_COMMON.name_tag,
+        priority=priority,
+        fulfilOnSubmit=fulfilOnSubmit,
+    )
+    if not ARGS_PARSER.dry_run:
+        work_requirement = CLIENT.work_client.add_work_requirement(work_requirement)
+        print_log(
+            f"Created "
+            f"{link_entity(CONFIG_COMMON.url, work_requirement)} "
+            f"({work_requirement.name})"
         )
-    )
-    print_log(
-        f"Created "
-        f"{link_entity(CONFIG_COMMON.url, work_requirement)} "
-        f"({work_requirement.name})"
-    )
+    else:
+        print_log("Dry-run: Printing Work Requirement:")
+        print_yd_object(work_requirement)
 
     # Keep track of uploaded files
     uploaded_files = []
@@ -179,7 +183,7 @@ def submit_work_requirement(
             if len(input_files_by_task_group[tg_number]) > 0:
                 print_log(f"Uploading files for Task Group '{task_group.name}'")
             for input_file in input_files_by_task_group[tg_number]:
-                if input_file not in uploaded_files:
+                if input_file not in uploaded_files and not ARGS_PARSER.dry_run:
                     upload_file(
                         client=CLIENT,
                         filename=input_file,
@@ -251,12 +255,12 @@ def create_task_group(
 
     # Name the Task Group
     num_task_groups = len(tasks_data[TASK_GROUPS])
-    task_group_name = get_name(
-        task_group_data,
+    num_tasks = len(task_group_data[TASKS])
+    task_group_name = get_task_group_name(
+        task_group_data.get(NAME, None),
         tg_number,
         num_task_groups,
-        "task_group_",
-        TASK_GROUP_NUMBER_SUB,
+        num_tasks,
     )
 
     # Assemble the RunSpecification values for the Task Group
@@ -403,6 +407,8 @@ def add_tasks_to_task_group(
                 "multiple Tasks in the group"
             )
 
+    num_task_groups = len(tasks_data[TASK_GROUPS])
+
     # Determine Task batching
     tasks = tasks_data[TASK_GROUPS][tg_number][TASKS]
     num_tasks = len(tasks) if task_count is None else task_count
@@ -423,7 +429,9 @@ def add_tasks_to_task_group(
         ):
             task_group_data = tasks_data[TASK_GROUPS][tg_number]
             task = tasks[task_number] if task_count is None else tasks[0]
-            task_name = get_name(task, task_number, num_tasks, "task_", TASK_NUMBER_SUB)
+            task_name = get_task_name(
+                task.get(NAME, None), task_number, num_tasks, tg_number, num_task_groups
+            )
             executable = check_str(
                 task.get(
                     EXECUTABLE,
@@ -556,20 +564,29 @@ def add_tasks_to_task_group(
                 )
             )
 
-        CLIENT.work_client.add_tasks_to_task_group_by_name(
-            CONFIG_COMMON.namespace,
-            work_requirement.name,
-            task_group.name,
-            tasks_list,
-        )
-
-        if num_task_batches > 1:
-            print_log(
-                f"Batch {str(batch_number + 1).zfill(len(str(num_task_batches)))} : "
-                f"Added {len(tasks_list):,d} "
-                f"Task(s) to Work Requirement Task Group '{task_group.name}'"
+        if not ARGS_PARSER.dry_run:
+            CLIENT.work_client.add_tasks_to_task_group_by_name(
+                CONFIG_COMMON.namespace,
+                work_requirement.name,
+                task_group.name,
+                tasks_list,
             )
-    print_log(f"Added a total of {num_tasks} Task(s) to Task Group '{task_group.name}'")
+        else:
+            print_log("Dry-run: Printing Tasks in Task Group")
+            print_tasks(task_group.name, tasks_list)
+
+        if not ARGS_PARSER.dry_run:
+            if num_task_batches > 1:
+                print_log(
+                    f"Batch {str(batch_number + 1).zfill(len(str(num_task_batches)))} : "
+                    f"Added {len(tasks_list):,d} "
+                    f"Task(s) to Work Requirement Task Group '{task_group.name}'"
+                )
+
+    if not ARGS_PARSER.dry_run:
+        print_log(
+            f"Added a total of {num_tasks} Task(s) to Task Group '{task_group.name}'"
+        )
 
 
 def follow_progress(work_requirement: WorkRequirement) -> None:
@@ -659,31 +676,77 @@ def check_for_duplicates_in_file_lists(
         )
 
 
-def get_name(
-    entity_data: Dict,
-    entity_number: int,
-    num_entities: int,
-    name_prefix: str,
-    substitution_str: str,
+def formatted_number_str(current_item_number: int, num_items: int) -> str:
+    """
+    Return a nicely formatted number string given a current item number
+    and a total number of items.
+    """
+    return str(current_item_number + 1).zfill(len(str(num_items)))
+
+
+def get_task_name(
+    name: Optional[str],
+    task_number: int,
+    num_tasks: int,
+    task_group_number: int,
+    num_task_groups: int,
 ) -> str:
     """
-    Create the name of an entity.
+    Create the name of a Task.
     Supports lazy substitution.
     """
 
-    def _entity_number_str():
-        return str(entity_number + 1).zfill(len(str(num_entities)))
-
-    try:
-        name = entity_data[NAME]
-        # Perform lazy substitutions
+    if name:
         name = name.replace(
-            f"{LAZY_SUBS_WRAPPER}{substitution_str}"
-            f"{LAZY_SUBS_WRAPPER}",
-            _entity_number_str(),
+            f"{LAZY_SUBS_WRAPPER}{TASK_NUMBER_SUB}{LAZY_SUBS_WRAPPER}",
+            formatted_number_str(task_number, num_tasks),
         )
-    except KeyError:
-        name = name_prefix + _entity_number_str()
+        name = name.replace(
+            f"{LAZY_SUBS_WRAPPER}{TASK_COUNT_SUB}{LAZY_SUBS_WRAPPER}",
+            str(num_tasks),
+        )
+        name = name.replace(
+            f"{LAZY_SUBS_WRAPPER}{TASK_GROUP_NUMBER_SUB}{LAZY_SUBS_WRAPPER}",
+            formatted_number_str(task_group_number, num_task_groups),
+        )
+        name = name.replace(
+            f"{LAZY_SUBS_WRAPPER}{TASK_GROUP_COUNT_SUB}{LAZY_SUBS_WRAPPER}",
+            str(num_task_groups),
+        )
+
+    else:
+        name = "task_" + formatted_number_str(task_number, num_tasks)
+
+    return name
+
+
+def get_task_group_name(
+    name: Optional[str],
+    task_group_number: int,
+    num_task_groups: int,
+    task_count: int,
+) -> str:
+    """
+    Create the name of a Task Group.
+    Supports lazy substitution.
+    """
+
+    if name:
+        name = name.replace(
+            f"{LAZY_SUBS_WRAPPER}{TASK_GROUP_NUMBER_SUB}{LAZY_SUBS_WRAPPER}",
+            formatted_number_str(task_group_number, num_task_groups),
+        )
+        name = name.replace(
+            f"{LAZY_SUBS_WRAPPER}{TASK_GROUP_COUNT_SUB}{LAZY_SUBS_WRAPPER}",
+            str(num_task_groups),
+        )
+        name = name.replace(
+            f"{LAZY_SUBS_WRAPPER}{TASK_COUNT_SUB}{LAZY_SUBS_WRAPPER}",
+            str(task_count),
+        )
+
+    else:
+        name = "task_group_" + formatted_number_str(task_group_number, num_task_groups)
 
     return name
 
@@ -728,7 +791,7 @@ def create_task(
     if task_type == "bash":
         if executable is None:
             raise Exception("No 'executable' specified for 'bash' Task Type")
-        if executable not in uploaded_files:
+        if executable not in uploaded_files and not ARGS_PARSER.dry_run:
             upload_file(
                 client=CLIENT,
                 filename=executable,
@@ -814,6 +877,10 @@ def create_task(
         # of the uploaded files, arguments or environment.
         pass
 
+    # Cannot use flatten_input_paths if there are no inputs
+    if inputs is None or len(inputs) == 0:
+        flatten_input_paths = None
+
     return Task(
         name=name,
         taskType=task_type,
